/******************************************************
 * $Workfile::                                       
 * $Author::                                         
 * $Date::                                           
 * $Revision::                                       
 *
 *
 * This header files defines the input/output
******************************************************/
`include "../src/define.v"
`include "../src/tb_define.v"

`timescale 1 ns / 10 ps

module spi_tiny_slave_model
  (
	input wire                  CLK,
	input wire                  RST_N,

  
	output reg                  SPI_DATA_OUT, //To external SPI bus
	input wire                  SPI_CLK, //From external SPI bus
    input wire                  SPI_DATA_IN, //From external SPI bus
	input wire                  SPI_CS_N,  //From external SPI bus
  
	input wire                  USB_RXF_N,
	input wire                  USB_TXE_N,
    inout wire  [7:0]           USB_DATA,

	output reg                  USB_RD_N,
	output reg                  USB_WR,
	output reg                  USB_SI
   );



   //----------------------------------------------
   // Parameter Declarations                        
   //----------------------------------------------
   parameter IDLE                   = 0,
	     RECEIVE_DATA				= 1,
		 SEND_TO_HOST			    = 2,
		 RECEIVE_COMPLETE			= 3,
		 WRITE_REGISTER 		    = 4,
		 LOAD_SHIFT_REG				= 5,
		 DELAY_WRITE_DATA_BYTE      = 6,
	     WRITE_DATA_BYTE			= 7,
	     WRITE_DATA_COMPLETE		= 8;
		 
   //----------------------------------------------
   // Bit Count Parameter Declarations                        
   //----------------------------------------------
   parameter BIT_COUNT_IDLE         = 0,
			 BIT_COUNT_CLK_HI		= 1,
			 BIT_COUNT_CLK_LOW		= 2,
			 BIT_COUNT_INC_BYTE		= 3;
			 
   parameter USB_RD_N_DELAY_COUNT   = 16'h4d;

   //------------------------------------------------
   // Wire and Register Declarations                
   //------------------------------------------------
   	reg [11:0]                 state, next;

    // Bit Count Registers
	reg [3:0]                  in_bit_count;
	reg [3:0]                  out_bit_count;
	
	// SPI Shift Registers
	reg [7:0]                  shift_in_reg, shift_out_reg;
	reg                        read_spi_data_in_once_flag;
	reg                        write_spi_data_out_once_flag;

	reg                        write_spi_data_out_response_once;

	reg                        load_shift_reg_high_byte;

	reg                        spi_clk_reg;
	
	// Storage Registers
	reg [7:0]                  transmit_byte_storage;
	reg [2:0]                  store_reg_state; 
	
	
	// in bit counter
	reg [1:0]                  bit_count_state;
	
	//Metastable Registers
	reg                        spi_test_reg;
	reg                        usb_rxf_reg;
	
	//USB_RD registers
	reg                        usb_rd_n_reg;
	reg [15:0]                 usb_rd_n_counter;
	

	//STATE_OUT Regisetrs
    reg [3:0]                  STATE_OUT;
   	
`ifdef SIM
   reg [8*26:1] state_name;
`endif

   //------------------------------------------------
   // Signal Assignments  
   //------------------------------------------------
   assign USB_DATA = state[RECEIVE_COMPLETE] ? shift_in_reg : 8'hz;
   //assign USB_RD_N = state[WRITE_REGISTER] ? 1'b0 : 1'b1;
   
   
   //-----------------------------------------------
   // Create a State Test Output Signal
   //-----------------------------------------------
/*   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  STATE_OUT <= 0;
	else
	  begin
	     if (state[IDLE])
           STATE_OUT <= 4'h0;
	     if (state[RECEIVE_DATA])
	       STATE_OUT <= 4'h1;
	     if (state[SEND_TO_HOST])
	       STATE_OUT <= 4'h2;
	     if (state[RECEIVE_COMPLETE])
	       STATE_OUT <= 4'h3;
	     if (state[WRITE_REGISTER])
	       STATE_OUT <= 4'h4;
	     if (state[LOAD_SHIFT_REG])
	       STATE_OUT <= 4'h5;
	     if (state[DELAY_WRITE_DATA_BYTE])
	       STATE_OUT <= 4'h6;
	     if (state[WRITE_DATA_BYTE])
	       STATE_OUT <= 4'h7;
	     if (state[WRITE_DATA_COMPLETE])
	       STATE_OUT <= 4'h8;
	  end // else: !if(!RST_N)
     end // always @ (posedge CLK or negedge RST_N)
*/
   //-----------------------------------------------
   //Register the USB_RXF_N input signal. 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	     usb_rxf_reg <= 1;
	else
	  begin
	     if (!USB_RXF_N)
	         usb_rxf_reg <= 0;
		 else
	         usb_rxf_reg <= 1;
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Assert the USB_RD_N input signal. 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	     USB_RD_N <= 1'b1;
		 usb_rd_n_reg <= 1'b0;
		 usb_rd_n_counter <= 0;
	end
	else
	  begin
	     if(state[WRITE_REGISTER])
	         USB_RD_N <= 1'b0;
		 else if(state[WRITE_DATA_COMPLETE] & !usb_rd_n_reg)
		 begin
	         USB_RD_N <= 1'b0;
			 usb_rd_n_reg <= 1'b1;
		 end
		 else if(usb_rd_n_reg)
		 begin
		     if(usb_rd_n_counter < USB_RD_N_DELAY_COUNT)
			    usb_rd_n_counter <= usb_rd_n_counter + 1'd1;
			 else
	         begin
	            USB_RD_N <= 1'b1;
	            usb_rd_n_reg <= 1'b0;
	            usb_rd_n_counter <= 0;
	         end
		 end
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // USB_WR output register based on state[RECEIVE_COMPLETE], 
   // !USB_TXE_N. 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	     USB_WR <= 1;
	else
	  begin
	     if (state[SEND_TO_HOST] && !USB_TXE_N )
	         USB_WR <= 0;
		 else
	         USB_WR <= 1;
	  end
     end // always @ (posedge CLK or negedge RST_N)


   //-----------------------------------------------
   // Store the byte for transmission from USB_DATA. 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	     transmit_byte_storage <= 0;
	else
	  begin
	     if(state[WRITE_REGISTER])
	         transmit_byte_storage <= USB_DATA;
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //-----------------------------------------------
   // Register the SPI_CLK input with spi_clk_reg. 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	 spi_clk_reg <= 0;
	else
	  begin
	     if ( SPI_CLK )
	       spi_clk_reg <= 1;
	     else 
	       spi_clk_reg <= 0;
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //------------------------------------------------
   // Input bit counter, maintains the count of serial
   // bits shifted in on SPI_MOSI
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  in_bit_count <= 0;
	  bit_count_state <= BIT_COUNT_IDLE;
	end
	else
	  begin
	  
	     case( bit_count_state )
		     BIT_COUNT_IDLE:
			     if ( state[RECEIVE_DATA] )
				     bit_count_state <= BIT_COUNT_CLK_HI;
			 BIT_COUNT_CLK_HI:
			     if( spi_clk_reg )
				 begin
				     bit_count_state <= BIT_COUNT_CLK_LOW;
	                 in_bit_count <= in_bit_count + 1;
				 end
			 BIT_COUNT_CLK_LOW:
			     if( !spi_clk_reg )
				     //bit_count_state <= BIT_COUNT_IDLE;
				     bit_count_state <= BIT_COUNT_CLK_HI;
				 else if( in_bit_count >= 8'h8)
				     bit_count_state <= BIT_COUNT_INC_BYTE;
			 BIT_COUNT_INC_BYTE:
			     if( state[RECEIVE_COMPLETE] )
				 begin
				     bit_count_state <= BIT_COUNT_IDLE;
	                 in_bit_count <= 0;
				 end
			 default:
			 begin
				     bit_count_state <= BIT_COUNT_IDLE;
			 end
		 endcase
	  end
     end

   //------------------------------------------------
   // Output bit counter keeps track of which bit is 
   // being trasnmitted
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
    begin
	   out_bit_count <= 0;
	    //spi_clk_reg <= 1'b0;
        write_spi_data_out_once_flag <= 1'b1;
	end
	else
	  begin
	     if ( state[WRITE_DATA_BYTE] && !spi_clk_reg && write_spi_data_out_once_flag )
	     begin
		    //spi_clk_reg <= 1'b1;
	       	out_bit_count <= out_bit_count + 1;
			write_spi_data_out_once_flag <= 0; 
	     end
		else if( state[WRITE_DATA_BYTE] && spi_clk_reg && !write_spi_data_out_once_flag )
 		begin 
		    //spi_clk_reg <= 1'b1;
        	write_spi_data_out_once_flag <= 1;
		end
	    else if ( state[WRITE_DATA_COMPLETE] )
		begin
		    //spi_clk_reg <= 1'b0;
	        out_bit_count <= 0;
		end
	  end
     end // always @ (posedge CLK or negedge RST_N)

   //------------------------------------------------
   // SPI input shift register
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
			shift_in_reg <= 0;
			read_spi_data_in_once_flag <= 1;
	end
	else
	  begin
	     //if ( state[RECEIVE_DATA] && !SPI_CLK && read_spi_data_in_once_flag )
	     if ( state[RECEIVE_DATA] && spi_clk_reg && read_spi_data_in_once_flag )
	       begin
		  shift_in_reg[0] <= SPI_DATA_IN;
		  shift_in_reg[7:1] <= shift_in_reg[6:0];
		  read_spi_data_in_once_flag <= 0;
	       end
		//else if ( state[RECEIVE_DATA] && SPI_CLK && !read_spi_data_in_once_flag )
		else if ( state[RECEIVE_DATA] && !spi_clk_reg && !read_spi_data_in_once_flag )
		begin
			read_spi_data_in_once_flag <= 1;
		end
		else if (state[WRITE_REGISTER])
			read_spi_data_in_once_flag <= 1;
	     else if (state[IDLE])
	       shift_in_reg <= 0;
	  end
     end // always @ (posedge CLK or negedge RST_N)
 
   //------------------------------------------------
   // SPI Shift Data Response out
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     shift_out_reg <= 8'h30;
	     SPI_DATA_OUT <= 1;
		 load_shift_reg_high_byte <= 0;
		 write_spi_data_out_response_once <= 0;
	  end
	else
	  begin

	     if (state[LOAD_SHIFT_REG] )
	     begin
	  		SPI_DATA_OUT <= 0;
			shift_out_reg <= transmit_byte_storage;
	     end
// Added new shift_out_reg <= register_value code
	     if (state[RECEIVE_DATA])
	     begin
		     SPI_DATA_OUT <= SPI_DATA_OUT;
		     shift_out_reg <= 0;
	     end
	     if (state[IDLE] )
	     begin
	  		SPI_DATA_OUT <= 0;
	     end
	     if ( state[WRITE_DATA_BYTE] )
	       begin

			if( spi_clk_reg  && write_spi_data_out_response_once )
			begin
		  		SPI_DATA_OUT <= shift_out_reg[7];
		  		shift_out_reg[7:1] <= shift_out_reg[6:0];
		 		shift_out_reg[0] <= 0;
				write_spi_data_out_response_once <= 0;
			end
			else if( !spi_clk_reg  && !write_spi_data_out_response_once )
			begin
				write_spi_data_out_response_once <= 1;
			end

	       end // if ((state[WRITE_BUSY_RESPONSE] || state[WRITE_CMD_RESPONSE]) &&...
	  end
     end // always @ (posedge CLK or negedge RST_N)   
   
	  
   //--------------------------------------------------
   // SPI State Machine
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     state <= 12'h00;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

   always @ ( /*AUTOSENSE*/SPI_CLK or SPI_DATA_IN or USB_WR
	     or in_bit_count or out_bit_count or state or usb_rxf_reg 
		 or spi_clk_reg or store_reg_state or bit_count_state
		 or USB_DATA or USB_RD_N or USB_RXF_N or USB_TXE_N)
     begin
	next = 12'h00;

	if (state[IDLE])
	begin
        if ( spi_clk_reg )
			next[RECEIVE_DATA] = 1'b1;
		else if(!usb_rxf_reg)
			next[WRITE_REGISTER] = 1'b1;
		else
			next[IDLE] = 1'b1;
	end


	if (state[RECEIVE_DATA])
	  begin
	     if ((in_bit_count >= 8) && spi_clk_reg)
	       next[SEND_TO_HOST] = 1'b1;
	     else
	       next[RECEIVE_DATA] = 1'b1;
	  end

	if (state[SEND_TO_HOST]) 
	  begin
	     if (USB_TXE_N)
			  next[RECEIVE_COMPLETE] = 1'b1;
	     else
	         next[SEND_TO_HOST] = 1'b1;
	  end 	     

	  if (state[RECEIVE_COMPLETE]) 
	  begin
	     if (!spi_clk_reg)
			  next[IDLE] = 1'b1;
	     else
	         next[RECEIVE_COMPLETE] = 1'b1;
	  end 	     

	  if (state[WRITE_REGISTER])
	  begin
	     if (!USB_RD_N)
	       next[LOAD_SHIFT_REG] = 1'b1;
	     else 
	       next[WRITE_REGISTER] = 1'b1;
	  end

	if (state[LOAD_SHIFT_REG])
	       next[DELAY_WRITE_DATA_BYTE] = 1'b1;

	if (state[DELAY_WRITE_DATA_BYTE])
	  begin
	       next[WRITE_DATA_BYTE] = 1'b1;
	  end

	if (state[WRITE_DATA_BYTE])
	  begin
	     if (( out_bit_count >= 8 ) && spi_clk_reg )
	       next[WRITE_DATA_COMPLETE] = 1'b1;
	     else
	       next[WRITE_DATA_BYTE] = 1'b1;
	  end

	if (state[WRITE_DATA_COMPLETE])
	  begin
	     if ( !spi_clk_reg )
	       next[IDLE] = 1'b1;
	     else
	       next[WRITE_DATA_COMPLETE] = 1'b1;
	  end

	
`ifdef SIM
	if (state == (1 << IDLE))
	  state_name = "IDLE";
	else if (state == (1 << RECEIVE_DATA))
	  state_name = "RECEIVE_DATA";
	else if (state == (1 << SEND_TO_HOST))
	  state_name = "SEND_TO_HOST";
	else if (state == (1 << RECEIVE_COMPLETE))
	  state_name = "RECEIVE_COMPLETE";
	else if (state == (1 << WRITE_REGISTER))
	  state_name = "WRITE_REGISTER";
	else if (state == (1 << LOAD_SHIFT_REG))
	  state_name = "LOAD_SHIFT_REG";
	else if (state == (1 << DELAY_WRITE_DATA_BYTE))
	  state_name = "DELAY_WRITE_DATA_BYTE";
	else if (state == (1 << WRITE_DATA_BYTE))
	  state_name = "WRITE_DATA_BYTE";
	else if (state == (1 << WRITE_DATA_COMPLETE))
	  state_name = "WRITE_DATA_COMPLETE";
`endif
     end // always @ (...
   

endmodule // spi_slave

